# Usage: niwatoko [OPTIONS] [FILE_PATH]

#   自然言語のソースコードを読み込んで実行するコマンドラインインターフェース。

#   Args:     file_path (str): 自然言語のソースコードが書かれたファイルのパス。     model (str):
#   使用するモデル（OpenAIまたはClaude）。     output (str): 生成されたコードの出力先ファイルのパス。     version
#   (bool): バージョン情報を表示するかどうか。

# Options:
#   -m, --model [openai|claude]  使用するモデルを選択します。
#   -o, --output PATH            生成されたコードの出力先ファイルを指定します。
#   -v, --version                バージョン情報を表示します。
#   --help                       Show this message and exit.



import subprocess
import os
from click.testing import CliRunner
from niwatoko.cli import main

class TestBase:
    """
    テストの共通処理を行うベースクラス
    """
    def setup_method(self):
        self.runner = CliRunner()

class TestVersion(TestBase):
    """
    バージョン情報が正しく表示されることを確認するテストケース
    """
    def test_version(self):
        result = self.runner.invoke(main, ['--version'])
        assert result.exit_code == 0
        assert 'niwatoko version:' in result.output

class TestNoFilePath(TestBase):
    """
    ファイルパスが指定されていない場合のエラーメッセージを確認するテストケース
    """
    def test_no_file_path(self):
        result = self.runner.invoke(main, [])
        assert result.exit_code == 0
        assert 'ファイルパスが指定されていません。' in result.output

class TestInvalidModel(TestBase):
    """
    無効なモデルが指定された場合のエラーを確認するテストケース
    """
    def test_invalid_model(self):
        result = self.runner.invoke(main, ['sample.md', '--model', 'invalid'])
        assert result.exit_code == 2
        assert 'Error: Invalid value for' in result.output

class TestGenerateResponse(TestBase):
    """
    自然言語コードから生成されたコードを確認するテストケースの基底クラス
    """
    def setup_method(self):
        super().setup_method()
        self.sample_file = None
        self.output_file = None

    def generate_response(self, model):
        result = self.runner.invoke(main, [str(self.sample_file), '--model', model, '--output', str(self.output_file)], input='n\n')
        assert '生成されたコードを' in result.output
        assert f'{str(self.output_file)} に書き出しました。実行しますか？(y/n)' in result.output
        assert result.exit_code == 0
        assert os.path.exists(self.output_file)
        assert 'に書き出しました。' in result.output

class TestGenerateResponseClaude(TestGenerateResponse):
    """
    Claudeモデルを使用して自然言語コードから生成されたコードを確認するテストケース
    """
    def test_generate_response_claude(self, tmp_path):
        self.sample_file = tmp_path / 'sample.md'
        self.sample_file.write_text('自然言語のコードサンプル')
        self.output_file = tmp_path / 'output.py'
        self.generate_response('claude')

class TestGenerateResponseOpenAI(TestGenerateResponse):
    """
    OpenAIモデルを使用して自然言語コードから生成されたコードを確認するテストケース
    """
    def test_generate_response_openai(self, tmp_path):
        self.sample_file = tmp_path / 'sample.md'
        self.sample_file.write_text('自然言語のコードサンプル')
        self.output_file = tmp_path / 'output.py'
        self.generate_response('openai')

# 上記をもとにリードミーを作成してください。特にオブジェクト思考のASCII図を描いてください。


import subprocess
import click
import os
from niwatoko.foundation_model.interpretation.llm.claude import generate_response
from niwatoko.foundation_model.interpretation.llm.gpt import generate_response as gpt_generate_response
import niwatoko
import re
from tqdm import tqdm
import itertools
import threading
import sys
import time

@click.command()
@click.argument('file_path', type=click.Path(exists=True), required=False)
@click.option('-m', '--model', type=click.Choice(['openai', 'claude']), default='claude', help='使用するモデルを選択します。')
@click.option('-o', '--output', type=click.Path(), help='生成されたコードの出力先ファイルを指定します。')
@click.option('-v', '--version',  is_flag=True, help='バージョン情報を表示します。')

def main(file_path, model, output, version):
    """
    自然言語のソースコードを読み込んで実行するコマンドラインインターフェース。

    Args:
        file_path (str): 自然言語のソースコードが書かれたファイルのパス。
        model (str): 使用するモデル（OpenAIまたはClaude）。
        output (str): 生成されたコードの出力先ファイルのパス。
        version (bool): バージョン情報を表示するかどうか。
    """
    if version:
        try:
            print(f"niwatoko version: {niwatoko.__version__}")
        except AttributeError:
            print("バージョン情報がniwatokoモジュールに存在しません。")
        return
    if not file_path:
        print("ファイルパスが指定されていません。")
        return

    processed_content = process_imports(file_path)

    print("実行中... (Processing...)")

    # ぐるぐるアニメーションを表示するスレッドを開始
    done = False
    spinner = threading.Thread(target=spin, args=(lambda: done,))
    spinner.start()

    if model == 'openai':
        generated_code = gpt_generate_response(
            model="gpt-4-turbo-2024-04-09",
            prompt=processed_content,
            max_tokens=1000,
            temperature=0.5,
        )
    elif model == 'claude':
        generated_code = generate_response(
            model='claude-3-haiku-20240307',
            prompt=processed_content,
            max_tokens=4000,
            temperature=0.2,
        )
    else:
        raise ValueError(f"無効なモデル: {model}")

    # ぐるぐるアニメーションを停止
    done = True
    spinner.join()

    if output:
        with open(output, 'w', encoding = "utf-8") as file:
            file.write(generated_code)
            print(f"生成されたコードを {output} に書き出しました。")
            print(f"Generated code has been written to {output}.")

def spin(done):
    """
    ぐるぐるアニメーションを表示する関数
    Args:
        done (function): アニメーションを停止するかどうかを判定する関数
    """
    for c in itertools.cycle(['|', '/', '-', '\\']):
        if done():
            break
        sys.stdout.write(f'\r{c}')
        sys.stdout.flush()
        time.sleep(0.1)
    sys.stdout.write('\rDone!     \n')

import os
def process_imports(file_path):
    with open(file_path, 'r') as file:
        lines = file.readlines()
    
    output = []
    for line in lines:
        output.extend(process_variable_imports(line))
        if line.startswith('- '):
            parts = line.strip().split(' = ')
            if len(parts) == 2:
                import_path = parts[1].strip()
                print('import_path:', import_path)
                if '[' in import_path and not import_path.startswith('['):
                    # ブラケットで囲まれたパスを抽出
                    path_within_brackets = import_path[import_path.index('[') + 1:import_path.index(']')]
                    # 拡張子を取得
                    extension = path_within_brackets.split('.')[-1]
                    # 拡張子に応じた処理を行う
                    if extension == 'md':
                        output.extend(process_md_import(import_path, line))
                    elif extension == 'py':
                        output.extend(process_py_import(import_path, line))
                    elif extension == 'rst':
                        output.extend(process_rst_import(import_path, line))
                    else:
                        # その他の拡張子の場合
                        output.extend(process_other_import(import_path, line))
                else:
                    # 拡張子が指定されていない場合、新しい関数を使用して処理
                    output.extend(process_no_extension_import(import_path, line))
            else:
                output.append(line)
        else:
            output.append(line)
    
    return ''.join(output)

def process_variable_imports(line):
    """
    {{}}で囲まれた変数のインポートを処理する関数

    Args:
        line (str): ファイルの行

    Returns:
        list: 処理後の出力行のリスト
    """
    output = []
    if '{{' in line and '}}' in line:
        # {{ }} で囲まれた変数を抽出
        variable = line[line.index('{{') + 2:line.index('}}')]
        # print(variable)
        # 変数名を小文字に変換し、拡張子.mdを追加
        import_path = variable.lower() + '.md'
        # 同一階層内のファイルの内容を取得
        import_content = get_file_content(import_path)
        output.extend([line, '```\n', import_content, '```\n'])
    else:
        output.append(line)
    return output
def process_no_extension_import(import_path, line):
    """
    Markdownファイルのインポートを処理する関数

    Args:
        import_path (str): インポートするファイルのパス
        line (str): インポート文の行

    Returns:
        list: 処理後の出力行のリスト
    """
    # print(import_path)
    import_path = import_path[1:-1]
    import_content = get_file_content(import_path)
    # print(import_content)
    return [line, '```\n', import_content, '```\n']


def process_md_import(import_path, line):
    """
    Markdownファイルのインポートを処理する関数

    Args:
        import_path (str): インポートするファイルのパス
        line (str): インポート文の行

    Returns:
        list: 処理後の出力行のリスト
    """
    import_path = import_path[4:-1] + '.md'  # 拡張子を追加
    import_content = get_file_content(import_path)
    return [line, '```\n', import_content, '```\n']

def process_py_import(import_path, line):
    """
    Pythonファイルのインポートを処理する関数

    Args:
        import_path (str): インポートするファイルのパス
        line (str): インポート文の行

    Returns:
        list: 処理後の出力行のリスト
    """
    import_path = import_path[4:-1] + '.py'  # 拡張子を追加
    import_content = get_file_content(import_path)
    return [line, '```python\n', import_content, '```\n']

def process_rst_import(import_path, line):
    """
    reStructuredTextファイルのインポートを処理する関数

    Args:
        import_path (str): インポートするファイルのパス
        line (str): インポート文の行

    Returns:
        list: 処理後の出力行のリスト
    """
    import_path = import_path[5:-1] + '.rst'  # 拡張子を追加
    import_content = get_file_content(import_path)
    return [line, '```rst\n', import_content, '```\n']

def process_other_import(import_path, line):
    """
    md, py, rst以外の拡張子のインポートを処理する関数

    Args:
        import_path (str): インポートするファイルのパス
        line (str): インポート文の行

    Returns:
        list: 処理後の出力行のリスト
    """
    extension = import_path.split('[')[0].strip()  # 文字列の一番左から [ の一文字前までを拡張子として取得
    import_path = import_path.split('[')[1].split(']')[0].strip() + '.' + extension  # [ と ] の間にあるパスに拡張子を追加
    import_content = get_file_content(import_path)
    return [line, f'```{extension}\n', import_content, '```\n']
def get_file_content(file_path):
    # print(file_path)
    if os.path.isfile(file_path):
        with open(file_path, 'r') as file:
            return file.read()
    else:
        error_message = f"ファイルが見つかりません: {file_path}"
        # print(error_message)
        raise FileNotFoundError(error_message)




# niwatoko - 自然言語プログラミング言語のPythonパッケージのsetup.pyファイルです。
# このファイルはパッケージのインストールや配布に必要な情報を含んでいます。

from setuptools import setup, find_packages

setup(
    name='niwatoko',
    version='1.1.5',
    description='自然言語でプログラミングを行うことができる新しいプログラミング言語',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='dai-motoki',
    author_email='dai.motoki1123@gmail.com',
    url='https://niwatoko2.vercel.app/',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'streamlit',
        'gradio',
        'openai',
        'anthropic',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: Japanese',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
    entry_points={
        'console_scripts': [
            'niwatoko=niwatoko.cli:main',
        ],
    },
    package_data={
        '': ['*.txt', '*.md', '*.json', '*.csv', '*.yaml', '*.yml'],
        'niwatoko': ['foundation_model/interpretation/llm/*', 
                     'grammar/*'],
    },
)



以下は、要件定義書に基づいて作成したpythonプログラムとpytestテストコードです。

```python
def add(a, b):
    """
    2つの数値を足し算する関数
    
    Args:
        a (int or float): 1つ目の数値
        b (int or float): 2つ目の数値
    
    Returns:
        int or float: aとbの和
    """
    if not isinstance(a, (int, float)) or not isinstance(b, (int, float)):
        raise TypeError("引数は数値でなければなりません")
    return a + b

def add_list(numbers):
    """
    数値のリストの合計を計算する関数
    
    Args:
        numbers (list): 数値のリスト
    
    Returns:
        int or float: numbersの合計値
    """
    if not all(isinstance(x, (int, float)) for x in numbers):
        raise TypeError("リストの要素は数値でなければなりません")
    return sum(numbers)

def add_multiple(*args):
    """
    複数の数値を足し算する関数
    
    Args:
        *args: 可変長引数。任意の数の数値を受け取る
    
    Returns:
        int or float: 引数として渡された数値の合計
    """
    if not all(isinstance(x, (int, float)) for x in args):
        raise TypeError("引数は数値でなければなりません")
    return sum(args)

def multiply(a, b):
    """
    2つの数値を掛け算する関数
    
    Args:
        a (int or float): 1つ目の数値
        b (int or float): 2つ目の数値
    
    Returns:
        int or float: aとbの積
    """
    if not isinstance(a, (int, float)) or not isinstance(b, (int, float)):
        raise TypeError("引数は数値でなければなりません")
    return a * b

def multiply_list(numbers):
    """
    数値のリストの積を計算する関数
    
    Args:
        numbers (list): 数値のリスト
    
    Returns:
        int or float: numbersの積
    """
    if not all(isinstance(x, (int, float)) for x in numbers):
        raise TypeError("リストの要素は数値でなければなりません")
    return prod(numbers)

def multiply_multiple(*args):
    """
    複数の数値を掛け算する関数
    
    Args:
        *args: 可変長引数。任意の数の数値を受け取る
    
    Returns:
        int or float: 引数として渡された数値の積
    """
    if not all(isinstance(x, (int, float)) for x in args):
        raise TypeError("引数は数値でなければなりません")
    return prod(args)

def prod(iterable):
    """
    iterable内の要素の積を計算する関数
    """
    result = 1
    for x in iterable:
        result *= x
    return result
```

```python
import pytest

def test_add():
    # 正常系
    assert add(2, 3) == 5
    assert add(2.5, 3.7) == 6.2
    assert add(2, 3.7) == 5.7
    
    # 異常系
    with pytest.raises(TypeError):
        add("2", 3)

def test_add_list():
    # 正常系
    assert add_list([1, 2, 3]) == 6
    assert add_list([2.5, 3.7, 4.8]) == 11.0
    assert add_list([1, 2.5, 3.7]) == 7.2
    
    # 異常系
    with pytest.raises(TypeError):
        add_list([1, 2, "3"])

def test_add_multiple():
    # 正常系
    assert add_multiple(1, 2, 3) == 6
    assert add_multiple(2.5, 3.7, 4.8) == 11.0
    assert add_multiple(1, 2.5, 3.7) == 7.2
    
    # 異常系
    with pytest.raises(TypeError):
        add_multiple(1, 2, "3")

def test_multiply():
    # 正常系
    assert multiply(2, 3) == 6
    assert multiply(2.5, 3.7) == 9.25
    assert multiply(2, 3.7) == 7.4
    
    # 異常系
    with pytest.raises(TypeError):
        multiply("2", 3)

def test_multiply_list():
    # 正常系
    assert multiply_list([1, 2, 3]) == 6
    assert multiply_list([2.5, 3.7, 4.8]) == 44.28
    assert multiply_list([1, 2.5, 3.7]) == 9.25
    
    # 異常系
    with pytest.raises(TypeError):
        multiply_list([1, 2, "3"])

def test_multiply_multiple():
    # 正常系
    assert multiply_multiple(1, 2, 3) == 6
    assert multiply_multiple(2.5, 3.7, 4.8) == 44.28
    assert multiply_multiple(1, 2.5, 3.7) == 9.25
    
    # 異常系
    with pytest.raises(TypeError):
        multiply_multiple(1, 2, "3")
```


import os
from openai import OpenAI
import niwatoko 
from dotenv import load_dotenv

load_dotenv()  # .envファイルから環境変数を読み込む
niwatoko_dir = os.path.dirname(niwatoko.__file__)                        # zoltraakパッケージのディレクトリパスを取得
with open(f"{niwatoko_dir}/grammar/system.md", "r", encoding = "utf-8") as f:
    system_prompt = f.read()
client = OpenAI(
    api_key=os.environ.get("OPENAI_API_KEY")
)


def generate_response(model, prompt, max_tokens, temperature):
    """
    OpenAI APIを使用してプロンプトに対する応答を生成する関数。

    Args:
        model (str): 使用するGPTモデルの名前（例: "gpt-4", "gpt-4-turbo"）。
        prompt (str): 応答を生成するためのプロンプト。
        max_tokens (int): 生成する最大トークン数。
        temperature (float): 生成時のランダム性を制御する温度パラメータ（0から1の範囲）。

    Returns:
        str: 生成された応答テキスト。
    """

    model = "gpt-4-turbo-2024-04-09"


    chat_completion = client.chat.completions.create(
        model=model,
        max_tokens=max_tokens,
        temperature=temperature,
        messages=[
            {"role": "system", "content": system_prompt},
            {"role": "user", "content": prompt}
        ]
    )

    return chat_completion.choices[0].message.content.strip()
